final def workDir = new File('.').canonicalFile
final def props = new Properties();
final def inputPropsFile = new File(args[0]);
try {
    inputPropsStream = new FileInputStream(inputPropsFile);
    props.load(inputPropsStream);
}
catch (IOException e) {
    throw new RuntimeException(e);
}

def propFileName = props['propFile']
def dirOffset = props['dir']?:'.'
def includes = props['includes']
def excludes = props['excludes']
def startDelimiter = props['startDelimiter']?:''
def endDelimiter = props['endDelimiter']?:''
def propertyPrefix = props['propertyPrefix']
def envPropValues = props['envPropValues']
def explicitTokens = props['explicitTokens']
def propFile = new File(propFileName)

try {

    def Properties properties = new Properties()
    def outStream  = new FileOutputStream(propFile)
    if(envPropValues) {
       if (propertyPrefix) {
            println "Looking for properties starting with $propertyPrefix"
        }
        envPropValues.split("(?<=(^|[^\\\\])(\\\\{2}){0,8}),").each { prop ->
            def index = prop.indexOf('=')
            def propName = prop.substring(0, index).trim().split("/")[-1]
            def propValue = index < (prop.length() - 1) ? prop.substring(index + 1) : ""
            propValue = propValue.replaceAll('\\\\,', ',')
            if (propValue.endsWith("\\\\")) {
                propValue = propValue.substring(0, propValue.length()-2) + "\\";
            }
            if ((!propertyPrefix || propName.startsWith(propertyPrefix))) {
                properties.setProperty(startDelimiter + propName + endDelimiter, propValue)
                println 'added: ' + startDelimiter + propName + endDelimiter + ':' + propValue
            }
        }
    }

    if (explicitTokens) {
        explicitTokens.eachLine {
            if (it && it.indexOf('->') > 0) {
                def index = it.indexOf('->')
                def propName = it.substring(0, index).trim()
                def propValue = index < (it.length() - 2) ? it.substring(index + 2) : ""
                properties.setProperty(propName, propValue)
                println 'added: ' + propName + ':' + propValue
            }
            else if (it) {
                println "Found invalid explicit token $it - missing -> separator"
            }
        }
    }
    properties.store(outStream, 'Auto generated property file')
    outStream.close()

    def ant = new AntBuilder()
    if (properties.size() > 0) {
        if (excludes) {
            ant.replace(
                    dir:dirOffset,
                    summary: 'true',
                    replacefilterfile: propFileName,
                    includes: includes,
                    excludes: excludes,
                    defaultexcludes: 'no')

        }
        else {
            ant.replace(
                    dir:dirOffset,
                    summary: 'true',
                    replacefilterfile: propFileName,
                    includes: includes,
                    defaultexcludes: 'no')
        }
    }
    else {
        println 'Did not find any properties or explicit tokens for replacement.'
    }
}
catch (Exception e) {
    e.printStackTrace()
    println "Error replacing tokens!"
    System.exit(1)
}
finally {
    if (propFile != null) {
        propFile.delete()
    }
}

System.exit(0)

